#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import atexit
from dput.changes import parse_changes_file
import logging
import os
import re
import shutil
import sys
import tempfile
import urllib2

from cupstream2distro import packagemanager


def parse_arguments():
    parser = argparse.ArgumentParser(
        description='Source package upload handler.')
    parser.add_argument('-p', '--ppa',
                        required=True,
                        help='Target PPA for source package upload.')
    parser.add_argument('-s', '--source',
                        required=True,
                        action='append',
                        help='A source package file to include in the upload.')
    return parser.parse_args()


def cleanup(directory):
    if os.path.exists(directory) and os.path.isdir(directory):
        logging.info('Removing temp directory: {}'.format(directory))
        shutil.rmtree(directory)


def create_temp_dir():
    directory = tempfile.mkdtemp()
    atexit.register(cleanup, directory)
    return directory


def get_url_contents(url):
    try:
        req = urllib2.Request(url)
        f = urllib2.urlopen(req)
        response = f.read()
        f.close()
        return response
    except IOError as e:
        raise EnvironmentError('Failed to open url [{}]: {}'.format(url, e))


def get_source_files(source_files):
    location = create_temp_dir()
    local_files = []
    for source in source_files:
        # Co-locate all the files into a temp directory
        file_name = os.path.basename(source)
        file_path = os.path.join(location, file_name)
        logging.info('Retrieving source file: {}'.format(file_name))
        with open(file_path, 'w') as out_file:
            out_file.write(get_url_contents(source))
        local_files.append(file_name)
    return (location, local_files)


def parse_dsc_file(dsc_file):
    regexp = re.compile("^Architecture: (.*)\n")
    with open(dsc_file) as in_file:
        for line in in_file:
            arch_lists = regexp.findall(line)
            if arch_lists:
                return arch_lists[0]


def parse_source_files(source_directory, source_files):
    package_list = []
    for source in source_files:
        source_name = os.path.join(source_directory, source)
        if source.endswith('changes'):
                changes = parse_changes_file(filename=source_name,
                                             directory=source_directory)
                package = {}
                package['name'] = changes.get('Source')
                package['version'] = changes.get('Version')
                package['files'] = []
                package['files'].append(changes.get_changes_file())
                for package_file in changes.get_files():
                    if os.path.exists(package_file):
                        if package_file.endswith('.dsc'):
                            package['architecture'] = parse_dsc_file(
                                package_file)
                        package['files'].append(package_file)
                    else:
                        raise EnvironmentError(
                            'Not found: {}'.format(package_file))
                package_list.append(package)
    return package_list


def upload_source_packages(ppa, upload_files):
    '''Attempts source file upload into the PPA.'''
    logging.info('Upload to the ppa: {}'.format(ppa))
    (source_directory, source_files) = get_source_files(upload_files)
    source_packages = parse_source_files(source_directory, source_files)
    for package in source_packages:
        packagemanager.upload_package(package['name'], package['version'],
                                      ppa, source_directory)
    # TODO Return the data set of packages uploaded
    return source_packages


def main():
    logging.basicConfig(level=logging.INFO, format="%(asctime)s %(levelname)s "
                        "%(message)s")

    args = parse_arguments()
    upload_source_packages(args.ppa, args.source)


if __name__ == '__main__':
    sys.exit(main())
